// =====================================================================
//  bintree.c
// =====================================================================

//  Author:         (c) 2013 Camil Demetrescu
//  License:        See the end of this file for license information
//  Created:        July 11, 2013

//  Last changed:   $Date: 2013/07/11 15:00:07 $
//  Changed by:     $Author: demetres $
//  Revision:       $Revision: 1.00 $

#include "bintree.h"
#include <stdlib.h>
#include <assert.h>

// ---------------------------------------------------------------------
// bintree_new
// --------------------------------------------------------------------
bintree* bintree_new(bintree* left, void* elem, bintree* right) {
	bintree* t = malloc(sizeof(bintree));
	assert(t != NULL);
	t->left  = left;
	t->elem  = elem;
	t->right = right;
	return t;
}


// ---------------------------------------------------------------------
// bintree_search
// --------------------------------------------------------------------
int bintree_search(bintree* root, void* key, int (*comp)(void* elem1, void* elem2)) {
	if (root == NULL) return 0;
	int res = comp(key, root->elem);
	if (res == 0) return 1;
	if (res <  0) return bintree_search(root->left, key, comp);
	return bintree_search(root->right, key, comp);
}


// ---------------------------------------------------------------------
// bintree_delete
// --------------------------------------------------------------------
void bintree_delete(bintree* root) {
	if (root == NULL) return;
	bintree_delete(root->left);
	bintree_delete(root->right);
	free(root);
}


// Copyright (C) 2013 Camil Demetrescu

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
// USA
