// =====================================================================
//  main.c
// =====================================================================

//  Author:         (c) 2013 Camil Demetrescu
//  License:        See the end of this file for license information
//  Created:        July 11, 2013

//  Last changed:   $Date: 2013/07/11 15:00:07 $
//  Changed by:     $Author: demetres $
//  Revision:       $Revision: 1.00 $


#include "bintree.h"
#include <stdio.h>
#include <string.h>


// ---------------------------------------------------------------------
// make_search_tree
// --------------------------------------------------------------------
bintree* make_search_tree() {
	return
		bintree_new(
			bintree_new(
				bintree_new(
                    bintree_new(
                        NULL,
                        "abstraction",
                        NULL
                    ),
                    "automorphism",
                    NULL
				),
				"boat",
				bintree_new(
                    bintree_new(
                        NULL,
                        "carrier",
                        NULL
                    ),
                    "cruise",
                    NULL
				)
			),
			"darwinism",
			bintree_new(
				bintree_new(
                    bintree_new(
                        NULL,
                        "drone",
                        NULL
                    ),
                    "ecosystem",
                    bintree_new(
                        NULL,
                        "fantasy",
                        NULL
                    )
				),
				"frontier",
				bintree_new(
                    bintree_new(
                        NULL,
                        "garage",
                        NULL
                    ),
                    "gratitude",
                    NULL
				)
			)	
		);
}


// ---------------------------------------------------------------------
// print_tree
// --------------------------------------------------------------------
void print_tree(bintree* root) {
    if (root == NULL) return;
    print_tree(root->left);
	printf("    %s\n", (char*)root->elem);
    print_tree(root->right);
}


// ---------------------------------------------------------------------
// comparator
// ---------------------------------------------------------------------
int comparator(void* a, void* b) {
    return strcmp(a, b);
}


// ---------------------------------------------------------------------
// check
// ---------------------------------------------------------------------
void check(bintree* t, char* key) {
    #ifdef SEARCH
    printf("   \"%s\" belongs to the tree? %s\n", key,
        bintree_search(t, key, comparator) ? "YES" : "NO");
    #endif
}


// ---------------------------------------------------------------------
// main
// ---------------------------------------------------------------------
int main() {

    // make sample tree
    bintree* t = make_search_tree();

    // display tree content
    printf("Tree content:\n");
    print_tree(t);

    // check search function
    #ifdef SEARCH
    printf("\nSearch test:\n");
    check(t, "drone");
    check(t, "roaming");
    check(t, "darwinism");
    check(t, "abnormal");
    check(t, "erosion");
    check(t, "automorphism");
    #endif

    // deallocate tree
    bintree_delete(t);

    return 0;
}


// Copyright (C) 2013 Camil Demetrescu

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
// USA
